from masz.helpers import parse_dt_from_json, parse_dt_to_json

class UserNote:
    def __init__(self, **kwargs) -> None:
        self.id = kwargs.get("id")
        self.guild_id = kwargs.get("guildId")
        self.user_id = kwargs.get("userId")
        self.description = kwargs.get("description")
        self.creator_id = kwargs.get("creatorId")
        self.updated_at = parse_dt_from_json(kwargs.get("updatedAt"))

    def __str__(self) -> str:
        return f"{self.user_id}: {self.description}"

    def to_dict(self) -> dict:
        return {
            "id": self.id,
            "guildId": self.guild_id,
            "userId": self.user_id,
            "description": self.description,
            "creatorId": self.creator_id,
            "updatedAt": parse_dt_to_json(self.updated_at)
        }



from platform import version


class Version:
    def __init__(self, **kwargs) -> None:
        self.masz_version = kwargs.get("version", '')
        self.wrapper_version = kwargs.get("wrapper_version", None)
        self.pre_release = bool(kwargs.get("pre_release"))

    def __str__(self) -> str:
        return f"API: {self.masz_version} | Wrapper: {self.wrapper_version}"



from masz.helpers import parse_dt_from_json

class Adminstats:
    def __init__(self, **kwargs) -> None:
        self.last_logins = kwargs.get("loginsInLast15Minutes", [])
        self.tracked_invites = kwargs.get("trackedInvites", 0)
        self.modCases = kwargs.get("modCases", 0)
        self.guilds = kwargs.get("guilds", 0)
        self.automod_events = kwargs.get("automodEvents", 0)
        self.usernotes = kwargs.get("userNotes", 0)
        self.usermappings = kwargs.get("userMappings", 0)
        self.apiTokens = kwargs.get("apiTokens", 0)
        self.next_cache = parse_dt_from_json(kwargs.get("nextCache", 0))
        self.cached_data = kwargs.get("cachedDataFromDiscord", [])



from typing import Union

from .console import console
from .masz_request import MASZRequestHandler
from .exceptions import *
from .obj import *


class MASZMotdAPI(Motd):
    def __init__(self, request_handler: MASZRequestHandler, guild_id: Union[str, int]) -> None:
        self.request_handler = request_handler
        self.guild_id = guild_id
        r = self.request_handler.request("GET", f"/guilds/{self.guild_id}/motd", handle_status_code=False)
        if r.status_code == 200:
            super().__init__(**r.json())

    def delete(self) -> bool:
        self.show_motd = False
        r = self.request_handler.request("PUT", f"/guilds/{self.guild_id}/motd", json_body=self.to_dict())
        if r.status_code == 200:
            super().__init__(**r.json())
        return r.status_code == 200

    def set(self, msg: str) -> bool:
        self.message = msg
        self.show_motd = True
        r = self.request_handler.request("PUT", f"/guilds/{self.guild_id}/motd", json_body=self.to_dict())
        if r.status_code == 200:
            super().__init__(**r.json())
        return r.status_code == 200

    def activate(self) -> bool:
        self.show_motd = True
        r = self.request_handler.request("PUT", f"/guilds/{self.guild_id}/motd", json_body=self.to_dict())
        if r.status_code == 200:
            super().__init__(**r.json())
        return r.status_code == 200

    def toggle(self) -> bool:
        self.show_motd = not self.show_motd
        r = self.request_handler.request("PUT", f"/guilds/{self.guild_id}/motd", json_body=self.to_dict())
        if r.status_code == 200:
            super().__init__(**r.json())
        return r.status_code == 200



from .role import DiscordRole

class DiscordGuild:
    def __init__(self, **kwargs) -> None:
        self.id = kwargs.get("id")
        self.name = kwargs.get("name")
        self.icon = kwargs.get("icon")
        self.roles = []
        for role in kwargs.get("roles", []):
            self.roles.append(DiscordRole(**role))
        self.image_url = f"https://cdn.discordapp.com/icons/{self.id}/{self.icon}.png" if self.icon else None

    def __str__(self) -> str:
        return self.name



from masz.helpers import parse_dt_from_json, parse_dt_to_json

class Comment:
    def __init__(self, **kwargs) -> None:
        self.id = kwargs.get("id")
        self.message = kwargs.get("message")
        self.user_id = kwargs.get("userId")
        self.created_at = parse_dt_from_json(kwargs.get("createdAt"))

    def __str__(self) -> str:
        return self.message

    def to_dict(self) -> dict:
        return {
            "id": self.id,
            "message": self.message,
            "userId": self.user_id,
            "createdAt": parse_dt_to_json(self.created_at)
        }



from masz.helpers import parse_dt_from_json, parse_dt_to_json

class UserMap:
    def __init__(self, **kwargs) -> None:
        self.id = kwargs.get("id")
        self.guild_id = kwargs.get("guildId")
        self.user_a = kwargs.get("userA")
        self.user_b = kwargs.get("userB")
        self.reason = kwargs.get("reason")
        self.creator_id = kwargs.get("creatorUserId")
        self.createdAt = parse_dt_from_json(kwargs.get("createdAt"))

    def __str__(self) -> str:
        return f"{self.user_a} - {self.user_b}: {self.reason}"

    def to_dict(self) -> dict:
        return {
            "id": self.id,
            "guildId": self.guild_id,
            "userA": self.user_a,
            "userB": self.user_b,
            "reason": self.reason,
            "creatorUserId": self.creator_id,
            "createdAt": parse_dt_to_json(self.createdAt)
        }


from requests import Response
import requests
from requests.exceptions import URLRequired

from .console import console
from .exceptions import *


class MASZRequestHandler:
    def __init__(self, url: str, token: str, api_version: int, header: str, header_prefix: str) -> None:
        self.url = url.rstrip('/')
        self.token = token
        self.api_version = api_version
        self.header = header
        self.header_prefix = header_prefix
        self.headers = {self.header: f"{self.header_prefix}{self.token}"}

    def request(self, method: str, resource: str, params: dict = dict(), headers: dict = dict(), handle_status_code: bool = True, json_body: dict = dict()) -> Response:
        try:
            if method == "GET":
                r = self.__get(resource, params, headers)
            if method == "GETSTATIC":
                r = self.__get_static(resource, params, headers)
            if method == "PUT":
                r = self.__put(resource, json_body, params, headers)
            if method == "POST":
                r = self.__post(resource, json_body, params, headers)
            if method == "DELETE":
                r = self.__delete(resource, params, headers)
            if r.status_code not in [200, 201]:
                console.verbose(f"[bright_red]{r.status_code}[/bright_red] [bright_cyan]{r.request.method}[/bright_cyan] [bright_magenta] /{resource.lstrip('/')}[/bright_magenta]")
            else:
                console.verbose(f"[bright_green]{r.status_code}[/bright_green][bright_magenta] [bright_cyan]{r.request.method}[/bright_cyan] /{resource.lstrip('/')}[/bright_magenta]")
        except Exception as e:
            raise MASZRequestFailure("Request failed", e)
        if r.status_code == 401 and handle_status_code:
            console.verbose(f"[bright_red]{r.status_code}[/bright_red][bright_magenta] {r.text}[/bright_magenta]")
            raise MASZLoginFailure("Unauthorized", r.text)
        if r.status_code not in [200, 201] and handle_status_code:
            console.verbose(f"[bright_red]{r.status_code}[/bright_red][bright_magenta] {r.text}[/bright_magenta]")
            raise MASZRequestFailure(f"Response code indicates {r.status_code}", r.text)
        return r

    def __get(self, resource: str, params: dict = dict(), headers: dict = dict()) -> Response:
        final_headers = {**headers, **self.headers}
        return requests.get(f"{self.url}/api/v{self.api_version}/{resource.lstrip('/')}", headers=final_headers, params=params)

    def __get_static(self, resource: str, params: dict = dict(), headers: dict = dict()) -> Response:
        final_headers = {**headers, **self.headers}
        return requests.get(f"{self.url}/{resource.lstrip('/')}", headers=final_headers, params=params)
    
    def __post(self, resource: str, body: dict, params: dict = dict(), headers: dict = dict()) -> Response:
        final_headers = {**headers, **self.headers}
        return requests.post(f"{self.url}/api/v{self.api_version}/{resource.lstrip('/')}", json=body, headers=final_headers, params=params)
    
    def __put(self, resource: str, body: dict, params: dict = dict(), headers: dict = dict()) -> Response:
        final_headers = {**headers, **self.headers}
        return requests.put(f"{self.url}/api/v{self.api_version}/{resource.lstrip('/')}", json=body, headers=final_headers, params=params)

    def __delete(self, resource: str, params: dict = dict(), headers: dict = dict()) -> Response:
        final_headers = {**headers, **self.headers}
        return requests.delete(f"{self.url}/api/v{self.api_version}/{resource.lstrip('/')}", headers=final_headers, params=params)



class MASZBaseException(Exception):
    def __init__(self, message, errors=None):            
        super().__init__(message)
            
        self.message = message
        self.errors = errors



from typing import Union, List

from .console import console
from .masz_request import MASZRequestHandler
from .masz_modcase import MASZModcaseAPI
from .masz_usernote import MASZUserNoteAPI
from .masz_usermap import MASZUserMapAPI
from .masz_motd import MASZMotdAPI
from .exceptions import *
from .obj import *


class MASZGuildAPI(GuildConfig):
    motd: MASZMotdAPI = None
    def __init__(self, request_handler: MASZRequestHandler, guild_id: Union[str, int]) -> None:
        self.request_handler = request_handler
        self.guild_id = guild_id
        r = self.request_handler.request("GET", f"/guilds/{guild_id}")
        super().__init__(**r.json())
        self.motd = MASZMotdAPI(request_handler, guild_id)

    def delete(self, delete_data: bool = False) -> bool:
        try:
            r = self.request_handler.request("DELETE", f"/guilds/{self.guild_id}", {'deleteData': delete_data})
        except MASZBaseException as e:
            console.verbose(f"Failed to delete guild {e}")
            return False
        return r.status_code == 200

    def update(self, **fields) -> bool:
        for k, v in fields.items():
            setattr(self, k, v)
        try:
            r = self.request_handler.request("PUT", f"/guilds/{self.guild_id}", json_body=self.to_dict())
        except MASZBaseException as e:
            console.verbose(f"Failed to update guild {e}")
            return False
        if r.status_code == 200:
            super().__init__(**r.json())
        return r.status_code == 200

    # Modcases
    # =================================================================================================================

    def get_modcase(self, case_id: Union[str, int]) -> MASZModcaseAPI:
        r = self.request_handler.request("GET", f"/guilds/{self.guild_id}/cases/{case_id}")
        return MASZModcaseAPI(self.request_handler, r.json())

    def get_modcases_paginated(self, start_page=0) -> List[MASZModcaseAPI]:
        r = self.request_handler.request("GET", f"/guilds/{self.guild_id}/cases", params={'startPage': start_page})
        return [MASZModcaseAPI(self.request_handler, x) for x in r.json()]

    def get_modcases(self) -> List[MASZModcaseAPI]:
        count = 0
        all_cases = []
        while True:
            new_cases = self.get_modcases_paginated(count)
            count += 1
            all_cases += new_cases
            if not len(new_cases):
                break
        return all_cases

    def create_modcase(self, modcase: Modcase, send_notification: bool = True, handle_punishment: bool = True, announce_dm: bool = True) -> MASZModcaseAPI:
        r = self.request_handler.request(
                "POST",
                f"/guilds/{self.guild_id}/cases",
                json_body=modcase.to_dict(),
                params={"sendNotification": send_notification, "handlePunishment": handle_punishment, "announceDm": announce_dm}
            )
        return MASZModcaseAPI(self.request_handler, r.json())

    def delete_modcase(self, case_id: Union[str, int], send_notification: bool = True, force_delete: bool = False) -> bool:
        r = self.request_handler.request("DELETE", f"/guilds/{self.guild_id}/cases/{case_id}",
                                            params={'sendNotification': send_notification, 'forceDelete': force_delete})
        return r.status_code == 200

    # UserNotes
    # =================================================================================================================

    def get_usernote(self, user_id: Union[str, int]) -> MASZUserNoteAPI:
        r = self.request_handler.request("GET", f"/guilds/{self.guild_id}/usernote/{user_id}")
        return MASZUserNoteAPI(self.request_handler, r.json())

    def get_usernotes(self) -> List[MASZUserNoteAPI]:
        r = self.request_handler.request("GET", f"/guilds/{self.guild_id}/usernote")
        return [MASZUserNoteAPI(self.request_handler, x) for x in r.json()]

    def create_usernote(self, usernote: UserNote) -> MASZUserNoteAPI:
        r = self.request_handler.request("PUT", f"/guilds/{self.guild_id}/usernote", json_body=usernote.to_dict())
        return MASZUserNoteAPI(self.request_handler, r.json())

    def delete_usernote(self, user_id: Union[str, int]) -> bool:
        r = self.request_handler.request("DELETE", f"/guilds/{self.guild_id}/usernote/{user_id}")
        return r.status_code == 200

    # UserMap
    # =================================================================================================================

    def get_usermaps_by_user(self, user_id: Union[str, int]) -> List[MASZUserMapAPI]:
        r = self.request_handler.request("GET", f"/guilds/{self.guild_id}/usermap/{user_id}")
        return [MASZUserMapAPI(self.request_handler, x) for x in r.json()]

    def get_usermaps(self) -> List[MASZUserMapAPI]:
        r = self.request_handler.request("GET", f"/guilds/{self.guild_id}/usermap")
        return [MASZUserMapAPI(self.request_handler, x) for x in r.json()]

    def create_usermap(self, usermap: UserMap) -> MASZUserMapAPI:
        usermap.guild_id = self.guild_id
        r = self.request_handler.request("POST", f"/guilds/{self.guild_id}/usermap", json_body=usermap.to_dict())
        console.log(r.json())
        return MASZUserMapAPI(self.request_handler, r.json())

    def delete_usermap(self, map_id: Union[str, int]) -> bool:
        r = self.request_handler.request("DELETE", f"/guilds/{self.guild_id}/usernote/{map_id}")
        return r.status_code == 200



from typing import List
from datetime import datetime

from masz.helpers import parse_dt_from_json, parse_dt_to_json
from .discord_user import DiscordUser
from ..exceptions import MASZInvalidResponse

class Motd:
    creator: DiscordUser

    def __init__(self, **kwargs) -> None:
        if "creator" in kwargs:  # may be null on PUT
            self.creator = DiscordUser(**kwargs["creator"])

        try:
            if "motd" in kwargs:
                motd = kwargs["motd"]
            elif "guildId" in kwargs:
                motd = kwargs
            else:
                raise KeyError("Invalid response")
        except KeyError as e:
            raise MASZInvalidResponse(e)
        
        self.id = motd.get("id")
        self.guild_id = motd.get("guildId")
        self.user_id = motd.get("userId")
        self.created_at = parse_dt_from_json(motd.get("createdAt"))
        self.message = motd.get("message")
        self.show_motd = motd.get("showMotd", False)

    def __str__(self) -> str:
        return f"{self.creator.username}#{self.creator.discriminator}: {self.message}" if self.show_motd else "Motd disabled"
    
    def to_dict(self) -> dict:
        return {
            "id": self.id,
            "guildId": self.guild_id,
            "userId": self.user_id,
            "createdAt": parse_dt_to_json(self.created_at),
            "message": self.message,
            "showMotd": self.show_motd
        }



from typing import Union, List

from .masz_request import MASZRequestHandler
from .exceptions import *
from .obj import *


class MASZDiscordAPI:
    def __init__(self, request_handler: MASZRequestHandler) -> None:
        self.request_handler = request_handler
    
    def get_current_user(self) -> AppUser:
        r = self.request_handler.request("GET", "/discord/users/@me")
        return AppUser(**r.json())

    def get_user(self, id: Union[str, int]) -> DiscordUser:
        r = self.request_handler.request("GET", f"/discord/users/{id}")
        return DiscordUser(**r.json())
    
    def get_guild(self, id: Union[str, int]) -> DiscordGuild:
        r = self.request_handler.request("GET", f"/discord/guilds/{id}")
        return DiscordGuild(**r.json())
    
    def get_guildchannels(self, id: Union[str, int]) -> List[DiscordChannel]:
        r = self.request_handler.request("GET", f"/discord/guilds/{id}/channels")
        return [DiscordChannel(**x) for x in r.json()]
    
    def get_guildmembers(self, id: Union[str, int]) -> List[DiscordMember]:
        r = self.request_handler.request("GET", f"/discord/guilds/{id}/members", {'partial': False})
        return [DiscordMember(**x) for x in r.json()]

    def get_partialguildmembers(self, id: Union[str, int]) -> List[DiscordUser]:
        r = self.request_handler.request("GET", f"/discord/guilds/{id}/members", {'partial': True})
        return [DiscordUser(**x) for x in r.json()]
